@@ -9,41 +9,52 @@
  *  -7 -> Message is too long
  *  -8 -> Substitution error during execution
  *  -9 -> 404 from server
+ * -10 -> Option not supported
+ * -11 -> Unknown reply
 */
 
 #ifndef _CONSOLE
 #ifdef __unix__
 #include "smssend.h"
-#else
-#include "mainsend.h"
+#else /* !__unix__ */
 #include "Unit1.h"
-#endif
-#else
+#endif /* __unix__ */
+#else /* _CONSOLE */
 #include "smssend.h"
-#include "smssend.h"
-#endif
+#endif /* !_CONSOLE */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
 
 #ifndef __unix__
 #define strcasecmp stricmp
 #define strncasecmp strnicmp
-#endif
+#endif /* !__unix__ */
+#ifdef _CONSOLE
+#define __unix__
+#endif /* _CONSOLE */
+
+#ifdef USE_PCRE
+static int PM_GetRegexMatch(char *data, char*regex, int gpos, int pos, char *buf, int blen);
+#endif /* USE_PCRE */
 
 char MsgBufTempo[40000];
 char BufTempo[40000];
 SMS_PProvider CurrentProviderRunning;
 int CurrentRunningPhase;
 SU_PList CurrentNoAdd;
-#ifdef __unix__
 char *CurrentScriptCheckUpdateName;
 char CurrentScriptCheckUpdateVersion[10];
 bool SMS_Quiet = false;
+#ifdef __unix__
 int DebugLevel = 0;
 SU_PList SMS_Alias = NULL;
-#else
+SU_PList SMS_Profiles = NULL; /* SU_PList */
+#else /* !__unix__ */
 int Sms_ErrorCode;
-#endif
+#endif /* __unix__ */
 
-#ifdef __unix__
 //---------------------------------------------------------------------------
 void CB_OnOkSmsSend(SU_PAnswer Ans,void *User)
 {
@@ -53,6 +64,7 @@ void CB_OnOkSmsSend(SU_PAnswer Ans,void *User)
       printf("A new version of SmsSend is available at http://zekiller.skytech.org/smssend_menu_en.html\n");
     else
       printf("You already have the latest version of SmsSend\n");
+    printf("You may now want to check you have the latest version of your favourite provider script with 'smssend <provider> -update'\n");
   }
 }
 
@@ -84,7 +96,7 @@ void CB_OnOk(SU_PAnswer Ans,void *User)
   pos2[0] = 0;
 #ifdef DEBUG
   printf("Your version : %s - Found version : %s\n",CurrentScriptCheckUpdateVersion,pos);
-#endif
+#endif /* DEBUG */
   if(strcmp(pos,CurrentScriptCheckUpdateVersion) <= 0)
   {
     if(!SMS_Quiet)
@@ -104,7 +116,7 @@ void CB_OnOk(SU_PAnswer Ans,void *User)
     snprintf(buf,sizeof(buf),"%s/.smssend/%s",getenv("HOME"),FileName);
     fp = fopen(buf,"wt");
     if(fp == NULL)
-#endif
+#endif /* !_WIN32 */
     {
       if(!SMS_Quiet)
         printf("SmsSend Error : Couldn't open %s. Aborting.\n",buf);
@@ -123,6 +135,41 @@ void CB_OnNotFound(SU_PAnswer Ans,void *User)
   if(!SMS_Quiet)
     printf("SmsSend Warning : Page not found\n");
 }
+void CB_OnOkInstall(SU_PAnswer Ans,void *User)
+{
+  FILE *fp;
+  char buf[1024];
+  char FileName[512];
+
+  snprintf(FileName,sizeof(FileName),"%s/%s",SMSSEND_SHAREPATH,CurrentScriptCheckUpdateName);
+  fp = fopen(FileName,"wt");
+  if(fp == NULL)
+  {
+#ifndef _WIN32
+    snprintf(buf,sizeof(buf),"%s/.smssend",getenv("HOME"));
+    mkdir(buf,0xFFFF);
+    snprintf(buf,sizeof(buf),"%s/.smssend/%s",getenv("HOME"),CurrentScriptCheckUpdateName);
+    fp = fopen(buf,"wt");
+    if(fp == NULL)
+#endif /* !_WIN32 */
+    {
+      if(!SMS_Quiet)
+        printf("SmsSend Error : Couldn't open %s. Aborting.\n",buf);
+      return;
+    }
+    if(!SMS_Quiet)
+      printf("SmsSend Warning : Couldn't open %s, saving new script to %s\n",FileName,buf);
+  }
+  fwrite(Ans->Data,1,Ans->Data_Length,fp);
+  fclose(fp);
+  if(!SMS_Quiet)
+    printf("Successfully installed new script %s\n",CurrentScriptCheckUpdateName);
+}
+void CB_OnNotFoundInstall(SU_PAnswer Ans,void *User)
+{
+  if(!SMS_Quiet)
+    printf("SmsSend Error : Script %s not found in SmsSend home page. Check script name\n",CurrentScriptCheckUpdateName);
+}
 //---------------------------------------------------------------------------
 void CheckForUpdateSmsSend(void)
 {
@@ -197,7 +244,37 @@ void CheckForUpdate(const char Name[])
     return;
   }
 }
-#endif
+
+void InstallScript(const char Name[])
+{
+  int ret;
+  SU_PList Exec;
+  SU_PHTTPActions Act;
+  char FileName[512];
+
+  CurrentScriptCheckUpdateName = (char *)Name;
+
+  Exec = NULL;
+  Act = (SU_PHTTPActions) malloc(sizeof(SU_THTTPActions));
+  memset(Act,0,sizeof(SU_THTTPActions));
+  Act->User = (void *)1;
+  Act->Command = ACT_GET;
+  SU_ExtractFileName(Name,FileName,sizeof(FileName));
+  snprintf(Act->URL,sizeof(Act->URL),"%s%s",SMSSEND_URL_SCRIPTS,FileName);
+  Act->CB.OnOk = CB_OnOkInstall;
+  Act->CB.OnNotFound = CB_OnNotFoundInstall;
+
+  Exec = SU_AddElementHead(Exec,Act);
+  ret = SU_ExecuteActions(Exec);
+  SU_FreeAction(Act);
+  SU_FreeList(Exec);
+  if(ret != 0)
+  {
+    if(!SMS_Quiet)
+      printf("SmsSend Error : Cannot connect\n");
+    return;
+  }
+}
 
 void RemoveComments(char *Ans,long int len)
 {
@@ -225,14 +302,15 @@ void RemoveComments(char *Ans,long int len)
 void CB_Execute_SendingCommand(SU_PHTTPActions Act)
 {
   SMS_PRunTime RT;
+  char *tt;
 
 #ifdef __unix__
   if(DebugLevel >= 1)
     printf("Executing Phase %d\n",CurrentRunningPhase);
-#else
+#else /* !__unix__ */
   Form1->StatusBar1->SimpleText = (AnsiString)"Executing phase "+IntToStr(CurrentRunningPhase)+"...";
   Sms_ErrorCode = 0;
-#endif
+#endif /* __unix__ */
 
   if(CurrentProviderRunning->Cookies != NULL)
   {
@@ -249,7 +327,7 @@ void CB_Execute_SendingCommand(SU_PHTTPActions Act)
 #ifdef __unix__
         if(DebugLevel >= 4)
           printf("Phase %d reached. Adding cookie %s\n",CurrentRunningPhase,C->Cookie->Name);
-#endif
+#endif /* __unix__ */
         SW_Cookies = SU_AddElementHead(SW_Cookies,C->Cookie);
         if(Ptr == CurrentProviderRunning->Cookies)
         {
@@ -268,14 +346,21 @@ void CB_Execute_SendingCommand(SU_PHTTPActions Act)
 
   RT = (SMS_PRunTime) malloc(sizeof(SMS_TRunTime));
   memset(RT,0,sizeof(SMS_TRunTime));
+  /* Split Params and URL */
+  tt = strchr(Act->URL,'?');
+  if((tt != NULL) && (Act->URL_Params == NULL))
+  {
+    tt[0] = 0;
+    Act->URL_Params = strdup(tt+1);
+  }
   RT->URL = strdup(Act->URL);
   if(Act->URL_Params != NULL)
     RT->Params = strdup(Act->URL_Params);
   RT->Host = strdup(Act->Host);
 #ifdef __unix__
   if(DebugLevel >= 4)
-    printf("Adding RunTime variables : %s&%s (%s)\n",RT->URL,(RT->Params == NULL)?"":RT->Params,RT->Host);
-#endif
+    printf("Adding RunTime variables : %s ? %s (%s)\n",RT->URL,(RT->Params == NULL)?"":RT->Params,RT->Host);
+#endif /* __unix__ */
   CurrentProviderRunning->RunTime = SU_AddElementTail(CurrentProviderRunning->RunTime,RT);
 }
 
@@ -288,7 +373,7 @@ void CB_Execute_Answer(SU_PAnswer Ans,void *User)
 #ifdef __unix__
   if(DebugLevel >= 4)
     printf("Adding RunTime data\n");
-#endif
+#endif /* __unix__ */
   RT = (SMS_PRunTime)SU_GetElementTail(CurrentProviderRunning->RunTime);
   if(RT == NULL)
   {
@@ -303,17 +388,30 @@ void CB_Execute_Answer(SU_PAnswer Ans,void *User)
   }
 }
 
+void CB_Execute_Other(SU_PAnswer Ans,int Code,void *User)
+{
+#ifdef __unix__
+  if(!SMS_Quiet)
+    printf("SmsSend Error : Untrapped reply from server (%d).... error in the script ? Contact the author of the script\n",Code);
+  exit(-11);
+#else /* !__unix__ */
+  Form1->StatusBar1->SimpleText = (AnsiString)"Error : Untrapped reply from server ("+IntToStr(Code)+").... error in the script ? Contact the author of the script";
+  Sms_ErrorCode = -11;
+  return;
+#endif /* __unix__ */
+}
+
 void CB_Execute_404(SU_PAnswer Ans,void *User)
 {
 #ifdef __unix__
   if(!SMS_Quiet)
-    printf("SmsSend Error : 404 answer from server.... error in the script ?\n");
+    printf("SmsSend Error : 404 answer from server.... error in the script ? Contact the author of the script\n");
   exit(-9);
-#else
-  Form1->StatusBar1->SimpleText = (AnsiString)"Error : 404 answer from server.... error in the script ?";
+#else /* !__unix__ */
+  Form1->StatusBar1->SimpleText = (AnsiString)"Error : 404 answer from server.... error in the script ? Contact the author of the script";
   Sms_ErrorCode = -9;
   return;
-#endif
+#endif /* __unix__ */
 }
 
 void CB_Execute_200(SU_PAnswer Ans,void *User)
@@ -348,17 +446,17 @@ void CB_Execute_200(SU_PAnswer Ans,void *User)
           if(!SMS_Quiet)
             printf("SmsSend Error : %s\n",S->Msg);
           exit(S->Error);
-#else
+#else /* !__unix__ */
           Form1->StatusBar1->SimpleText = (AnsiString)"Error : " + (AnsiString)S->Msg;
           Sms_ErrorCode = S->Error;
           return;
-#endif
+#endif /* __unix__ */
         }
         else
         {
 #ifdef __unix__
           if(!SMS_Quiet)
-#endif
+#endif /* __unix__ */
             printf("Result : %s\n",S->Msg);
         }
         Found = 1;
@@ -375,17 +473,17 @@ void CB_Execute_200(SU_PAnswer Ans,void *User)
               if(!SMS_Quiet)
                 printf("SmsSend Error : %s\n",S->Msg);
               exit(S->Error);
-#else
+#else /* !__unix__ */
               Form1->StatusBar1->SimpleText = (AnsiString)"Error : " + (AnsiString)S->Msg;
               Sms_ErrorCode = S->Error;
               return;
-#endif
+#endif /* __unix__ */
             }
             else
             {
 #ifdef __unix__
               if(!SMS_Quiet)
-#endif
+#endif /* __unix__ */
                 printf("Result : %s\n",S->Msg);
             }
             Found = 1;
@@ -409,11 +507,11 @@ char *CheckMessage(char Msg[],const int LenMax)
     if(!SMS_Quiet)
       printf("Message too long : %d caracters max (yours is %d long)\n",LenMax,len);
     exit(-7);
-#else
+#else /* !__unix__ */
     Form1->StatusBar1->SimpleText = "Message too long : "+IntToStr(LenMax)+" caracters max (yours is "+IntToStr(len)+" long)";
     Sms_ErrorCode = -7;
     return NULL;
-#endif
+#endif /* __unix__ */
   }
   pos = 0;
   for(i=0;i<len;i++)
@@ -538,7 +636,7 @@ void FreeProvider(SMS_PProvider Pv)
 #ifndef __unix__
   if(Pv->Path != NULL)
     free(Pv->Path);
-#endif
+#endif /* __unix__ */
   free(Pv);
 }
 
@@ -565,7 +663,7 @@ SMS_PProvider LoadProviderFile(const char FileName[])
   memset(Pv,0,sizeof(SMS_TProvider));
 #ifndef __unix__
   Pv->Path = strdup(FileName);
-#endif
+#endif /* __unix__ */
   NbParams = 0;
   Group = 0;
   SearchPtr = NULL;
@@ -625,7 +723,7 @@ SMS_PProvider LoadProviderFile(const char FileName[])
         FreeProvider(Pv);
         return NULL;
       }
-#endif
+#endif /* !__unix__ */
     }
     else if(Name[0] == '$')
     {
@@ -641,7 +739,7 @@ SMS_PProvider LoadProviderFile(const char FileName[])
 #ifdef __unix__
             if(DebugLevel >= 4)
               printf("Adding alias : %s <-> %s\n",Name+1,Value);
-#endif
+#endif /* __unix__ */
             Pv->Params[i].Alias = strdup(Value);
             found = 1;
           }
@@ -780,6 +878,7 @@ SMS_PProvider LoadProviderFile(const char FileName[])
           Act->User = AU;
           SearchPtr = NULL;
           NoAddPtr = NULL;
+          Act->CB.OnOtherReply = CB_Execute_Other;
           Act->CB.OnNotFound = CB_Execute_404;
           Act->CB.OnOk = CB_Execute_200;
         }
@@ -856,7 +955,7 @@ SMS_PProvider LoadProviderFile(const char FileName[])
 #ifdef __unix__
       if(DebugLevel >= 4)
         printf("Adding cookie %s=%s for %s%s at phase %d\n",Cookie->Name,Cookie->Value,Cookie->Domain,Cookie->Path,Phase);
-#endif
+#endif /* __unix__ */
     }
     else
     {
@@ -883,7 +982,7 @@ char *GetRunTimeValue(SMS_PProvider Pv,char *Num,int Type)
 #ifdef __unix__
   if(DebugLevel >= 4)
     printf("Getting run time value from phase %d\n",val);
-#endif
+#endif /* __unix__ */
   RT = (SMS_PRunTime)SU_GetElementPos(Pv->RunTime,val-1); /* First Run time is 1, but first SU_PList element is 0 */
   if(RT == NULL)
   {
@@ -904,7 +1003,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
 {
   int i,pos,found,do_it,read,j;
   char *p,*str=NULL,*S,*saf,*hst;
-  char *data,*q;
+  char *data,*q,*r;
   SU_PHTTPActions Act = NULL;
   char buf[20000];
   char fbuf[40000];
@@ -914,6 +1013,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
   SU_PForm Form;
   SU_PList Ptr,Ptr2,InputGet = NULL;
   FILE *fp_out;
+  char tbuf[1024];
 
   saf = strdup(Strng);
   S = saf;
@@ -990,6 +1090,45 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
             }
           }
         }
+        else if(strncasecmp(S,"RTRegex-",8) == 0)
+        {
+#ifdef USE_PCRE
+          q = strchr(S+8,'-');
+          if(q != NULL)
+          {
+            q[0] = 0;q++;
+            data = GetRunTimeValue(Pv,S+8,RUNTIME_DATA);
+            if(data != NULL) {
+              r = strchr(q,'-');
+              if (r != NULL) {
+                int gpos; char *s;
+                r[0] = 0;r++;
+                gpos = atoi(q);
+                s = strchr(r,'-');
+                if (s != NULL) {
+                  int pos;
+                  s[0] = 0;s++;
+                  pos = atoi(r);
+                  if (PM_GetRegexMatch(data,s,gpos,pos,buf,sizeof(buf)) == 0) {
+                    found = 0;
+                    str = buf;
+		  }
+		}
+              }
+            }
+          }
+#else /* !USE_PCRE */
+#ifdef __unix__
+          if(!SMS_Quiet)
+            printf("Option RTRegex not supported by this version of smssend (try to recompile with pcre support)\n");
+          exit(-10);
+#else /* !__unix__ */
+          Form1->StatusBar1->SimpleText = "Option RTRegex not supported by this version of smssend (try to recompile with pcre support)";
+          Sms_ErrorCode = -10;
+          return NULL;
+#endif /* __unix__ */
+#endif /* USE_PCRE */
+        }
         else if(strncasecmp(S,"RTGetInput-",11) == 0)
         {
           q = strchr(S+11,'-');
@@ -1018,6 +1157,70 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
           }
         }
         else if(strncasecmp(S,"RTGetInput2-",12) == 0)
+        {
+          q = strchr(S+12,'-');
+          if(q != NULL)//      q     r
+          {//RTGetInput2-<num>-<idx>-<flags>
+            q[0] = 0;q++;
+            data = GetRunTimeValue(Pv,S+12,RUNTIME_DATA);
+            if(data != NULL)
+            {
+              r = strchr(q,'-');
+              if(r != NULL)
+              {
+                r[0] = 0;r++;
+                i = 1;
+                j = atoi(q);
+                In = SU_GetInput(data);
+                while(In != NULL)
+                {
+                  if(i == j)
+                  {
+                    InputGet = SU_AddElementHead(InputGet,strdup(In->Name));
+                    found = 0;
+                    switch(*r)
+                    {
+                      case 'A' : /* want name and value */
+                        snprintf(buf,sizeof(buf),"\"%s\"=\"%s\"",In->Name,In->Value);break;
+                      case 'a' : /* want name and value */
+                        snprintf(buf,sizeof(buf),"%s=%s",In->Name,In->Value);break;
+                      case 'N' : /* want only name */
+                        snprintf(buf,sizeof(buf),"\"%s\"",In->Name);break;
+                      case 'n' : /* want only name */
+                        snprintf(buf,sizeof(buf),"%s",In->Name);break;
+                      case 'V' : /* want only value */
+                        snprintf(buf,sizeof(buf),"\"%s\"",In->Value);break;
+                      case 'v' : /* want only value */
+                        snprintf(buf,sizeof(buf),"%s",In->Value);break;
+                      case 'T' : /* want only type */
+                        snprintf(buf,sizeof(buf),"\"%s\"",In->Type);break;
+                      case 't' : /* want only type */
+                        snprintf(buf,sizeof(buf),"%s",In->Type);break;
+                      default :
+#ifdef __unix__
+                        printf("SmsSend Error in Execute : Unknown flag in RTGetInput2 : %c\n",*r);
+                        free(saf);
+                        exit(-8);
+#else /* !__unix__ */
+                        Form1->StatusBar1->SimpleText = (AnsiString)"Error in Execute : Unknown flag in RTGetInput2";
+                        free(saf);
+                        Sms_ErrorCode = -8;
+                        return NULL;
+#endif /* __unix__ */
+                    }
+                    str = buf;
+                    SU_FreeInput(In);
+                    break;
+                  }
+                  SU_FreeInput(In);
+                  In = SU_GetNextInput();
+                  i++;
+                }
+              }
+            }
+          }
+        }
+        else if(strncasecmp(S,"RTGetInput3-",12) == 0)
         {
           q = strchr(S+12,'-');
           if(q != NULL)
@@ -1026,23 +1229,33 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
             data = GetRunTimeValue(Pv,S+12,RUNTIME_DATA);
             if(data != NULL)
             {
-              i = 1;
-              j = atoi(q);
-              In = SU_GetInput(data);
-              while(In != NULL)
+              r = strchr(q,'-');
+              if(r != NULL)
               {
-                if(i == j)
+                r[0] = 0;r++;
+                Form = SU_RetrieveForm(data,atoi(q));
+                if(Form != NULL)
                 {
-                  InputGet = SU_AddElementHead(InputGet,strdup(In->Name));
-                  found = 0;
-                  snprintf(buf,sizeof(buf),"\"%s\"=\"%s\"",In->Name,In->Value);
-                  str = buf;
-                  SU_FreeInput(In);
-                  break;
+                  Ptr = Form->Inputs;
+                  i = 1;
+                  j = atoi(r);
+                  while(Ptr != NULL)
+                  {
+                    if(i == j)
+                    {
+                      In = (SU_PInput)Ptr->Data;
+                      InputGet = SU_AddElementHead(InputGet,strdup(In->Name));
+                      found = 0;
+                      snprintf(buf,sizeof(buf),"\"%s\"=\"%s\"",In->Name,In->Value);
+                      str = buf;
+                      SU_FreeForm(Form);
+                      break;
+                    }
+                    Ptr = Ptr->Next;
+                    i++;
+                  }
+                  SU_FreeForm(Form);
                 }
-                SU_FreeInput(In);
-                In = SU_GetNextInput();
-                i++;
               }
             }
           }
@@ -1061,13 +1274,21 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
               {
                 if(strncmp(Form->Action,"http://",7) != 0) /* If Action is relative */
                 {
-                  str = GetRunTimeValue(Pv,S+13,RUNTIME_URL);
-                  hst = GetRunTimeValue(Pv,S+13,RUNTIME_HOST);
-                  if((str != NULL) && (hst != NULL))
+                  if(strncmp(Form->Action,"https://",8) != 0) /* If Action is REALLY relative */
                   {
-                    str = SU_AddLocationToUrl(str,hst,Form->Action);
-                    SU_strcpy(fbuf,str,sizeof(fbuf));
-                    free(str);
+                    str = GetRunTimeValue(Pv,S+13,RUNTIME_URL);
+                    hst = GetRunTimeValue(Pv,S+13,RUNTIME_HOST);
+                    if((str != NULL) && (hst != NULL))
+                    {
+                      str = SU_AddLocationToUrl(str,hst,Form->Action,strncasecmp(str,"https://",8) == 0);
+                      SU_strcpy(fbuf,str,sizeof(fbuf));
+                      free(str);
+                      found = 0;
+                    }
+                  }
+                  else
+                  {
+                    SU_strcpy(fbuf,Form->Action,sizeof(fbuf));
                     found = 0;
                   }
                 }
@@ -1110,7 +1331,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
 #ifdef __unix__
                         if(DebugLevel >= 4)
                           printf("Not adding %s input, because already added by script\n",In->Name);
-#endif
+#endif /* __unix__ */
                         do_it = false;
                         break;
                       }
@@ -1126,10 +1347,10 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
                       {
                         if(strcasecmp((char *)Ptr2->Data,In->Name) == 0)
                         {
-  #ifdef __unix__
+#ifdef __unix__
                           if(DebugLevel >= 4)
                             printf("Not adding %s input, because set to NoAdd state by script\n",In->Name);
-  #endif
+#endif /* __unix__ */
                           do_it = false;
                           break;
                         }
@@ -1142,7 +1363,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
 #ifdef __unix__
                     if(DebugLevel >= 4)
                       printf("Adding %s input from form %d\n",In->Name,atoi(q));
-#endif
+#endif /* __unix__ */
                     if(fbuf[0] != 0)
                       SU_strcat(fbuf,"&",sizeof(fbuf));
                     if(strcasecmp(In->Type,"image") == 0)
@@ -1202,7 +1423,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
                     snprintf(cmd,sizeof(cmd),"more %s | %s > %s",tmpname2,q,tmpname);
 #ifdef DEBUG
                     printf("Executing command : %s\n",cmd);
-#endif
+#endif /* DEBUG */
                     system(cmd);
                     fp_out = fopen(tmpname,"rb");
                     if(fp_out != NULL)
@@ -1218,7 +1439,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
                       }
 #ifdef DEBUG
                       printf("Returned buffer : %s\n",fbuf);
-#endif
+#endif /* DEBUG */
                       str = fbuf;
                       found = 0;
                     }
@@ -1247,7 +1468,7 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
                 SU_strcpy(buf,str,sizeof(buf));
 #ifdef DEBUG
                 printf("String found : %s\n",buf);
-#endif
+#endif /* DEBUG */
                 free(str);
                 str = buf;
               }
@@ -1300,13 +1521,20 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
 #ifdef DEBUG
                 if(found == 0)
                   printf("SubURL found : %s\n",str);
-#endif
+#endif /* DEBUG */
               }
             }
           }
         }
         else
         {
+          char *dum = strchr(S,'-'); /* Search for sub-string */
+
+          if(dum != NULL)
+          {
+            dum[0] = 0;
+            dum++;
+          }
           for(i=0;i<Pv->NbParams;i++)
           {
             if(strcasecmp(S,Pv->Params[i].Name) == 0)
@@ -1323,22 +1551,44 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
           {
             SU_FreeListElem(InputGet);
 #ifdef __unix__
-            printf("SmsSend Error in Execute : Substitution name not found in params : %s\n",S);
+            printf("SmsSend Error in Execute : Substitution name not found in params : %s (error in the script ? Contact the author of the script)\n",S);
             free(saf);
             exit(-8);
           }
-#else
-            Form1->StatusBar1->SimpleText = (AnsiString)"Error in Execute : Substitution name not found in params : " + (AnsiString)S;
+#else /* !__unix__ */
+            Form1->StatusBar1->SimpleText = (AnsiString)"Error in Execute : Substitution name not found in params : " + (AnsiString)S + " (error in the script ? Contact the author of the script)";
             free(saf);
             Sms_ErrorCode = -8;
             return NULL;
           }
           Sms_ErrorCode = 0;
-#endif
-          if(Pv->Params[found].Convert || (Pv->Params[found].Size != 0))
-            str = CheckMessage(Pv->Params[found].Value,(Pv->Params[found].Size == 0)?512:Pv->Params[found].Size);
+#endif /* __unix__ */
+          if(dum != NULL)
+          {
+            char *dum2 = strchr(dum,'-');
+            int vv,xx;
+            if(dum2 == NULL)
+              SU_strcpy(tbuf,Pv->Params[found].Value,sizeof(tbuf));
+            else
+            {
+              dum2[0] = 0;
+              dum2++;
+              xx = atoi(dum);
+              if(xx >= strlen(Pv->Params[found].Value))
+                xx = 0;
+              vv = atoi(dum2) + 1;
+              if(vv > sizeof(tbuf))
+                vv = sizeof(tbuf);
+              SU_strcpy(tbuf,Pv->Params[found].Value+xx,vv);
+            }
+          }
           else
-            str = Pv->Params[found].Value;
+            SU_strcpy(tbuf,Pv->Params[found].Value,sizeof(tbuf));
+
+          if(Pv->Params[found].Convert || (Pv->Params[found].Size != 0))
+            str = CheckMessage(tbuf,(Pv->Params[found].Size == 0)?512:Pv->Params[found].Size);
+          else
+            str = tbuf;
 #ifndef __unix__
           if(Sms_ErrorCode != 0)
           {
@@ -1346,22 +1596,22 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
             free(saf);
             return NULL;
           }
-#endif
+#endif /* !__unix__ */
         }
       }
       if(found == -1)
       {
         SU_FreeListElem(InputGet);
 #ifdef __unix__
-        printf("SmsSend Error in Execute : Substitution name not found in params : %s\n",S);
+        printf("SmsSend Error in Execute : Substitution name not found in params : %s (error in the script ? Contact the author of the script)\n",S);
         free(saf);
         exit(-8);
-#else
-        Form1->StatusBar1->SimpleText = (AnsiString)"Error in Execute : Substitution name not found in params : " + (AnsiString)S;
+#else /* !__unix__ */
+        Form1->StatusBar1->SimpleText = (AnsiString)"Error in Execute : Substitution name not found in params : " + (AnsiString)S + " (error in the script ? Contact the author of the script)";
         free(saf);
         Sms_ErrorCode = -8;
         return NULL;
-#endif
+#endif /* __unix__ */
       }
       strcpy(&BufTempo[pos],str);
       pos = strlen(BufTempo);
@@ -1381,9 +1631,9 @@ char *TranslateString(char *Strng,SMS_PProvider Pv)
 
 #ifdef __unix__
 void ExecuteProvider(SMS_PProvider Pv)
-#else
+#else /* !__unix__ */
 int ExecuteProvider(SMS_PProvider Pv)
-#endif
+#endif /* __unix__ */
 {
   SU_PList Exec,Read;
   SU_PHTTPActions Act;
@@ -1412,68 +1662,146 @@ int ExecuteProvider(SMS_PProvider Pv)
     {
 #ifdef __unix__
       Act->Referer = strdup(TranslateString(Act->Referer,Pv));
-#else
+#else /* !__unix__ */
       tmp = TranslateString(Act->Referer,Pv);
       if(tmp == NULL)
         return Sms_ErrorCode;
       Act->Referer = strdup(tmp);
-#endif
+#endif /* __unix__ */
     }
     Exec = NULL;
     tmp = TranslateString(Act->URL,Pv);
 #ifndef __unix__
     if(tmp == NULL)
       return Sms_ErrorCode;
-#endif
+#endif /* !__unix__ */
     SU_strcpy(Act->URL,tmp,sizeof(Act->URL));
     if(Act->URL_Params != NULL)
     {
 #ifdef __unix__
       Act->URL_Params = strdup(TranslateString(Act->URL_Params,Pv));
-#else
+#else /* !__unix__ */
       tmp = TranslateString(Act->URL_Params,Pv);
       if(tmp == NULL)
         return Sms_ErrorCode;
       Act->URL_Params = strdup(tmp);
-#endif
+#endif /* __unix__ */
     }
     if(Act->Post_Data != NULL)
     {
 #ifdef __unix__
       Act->Post_Data = strdup(TranslateString(Act->Post_Data,Pv));
-#else
+#else /* !__unix__ */
       tmp = TranslateString(Act->Post_Data,Pv);
       if(tmp == NULL)
         return Sms_ErrorCode;
       Act->Post_Data = strdup(tmp);
-#endif
+#endif /* __unix__ */
       Act->Post_Length = strlen(Act->Post_Data);
     }
 
     Exec = SU_AddElementHead(Exec,Act);
 #ifndef __unix__
     Sms_ErrorCode = 0;
-#endif
+#endif /* !__unix__ */
     ret = SU_ExecuteActions(Exec);
     SU_FreeAction(Act);
     SU_FreeList(Exec);
 #ifdef __unix__
     if(ret != 0)
       exit(ret);
-#else
+#else /* !__unix__ */
     if(ret != 0)
       return ret;
     if(Sms_ErrorCode != 0)
       return Sms_ErrorCode;
-#endif
+#endif /* __unix__ */
     Read = Read->Next;
   }
 #ifndef __unix__
   return 0;
-#endif
+#endif /* !__unix__ */
 }
 
 #ifdef __unix__
+SU_PList GetProfile(const char ProfileName[])
+{
+  SU_PList Ptr = SMS_Profiles;
+
+  while(Ptr != NULL)
+  {
+    if(SU_strcasecmp(ProfileName,(char *)(((SU_PList)(Ptr->Data))->Data)))
+    {
+      if(DebugLevel >= 1)
+        printf("Using Profile '%s'\n",ProfileName);
+      return Ptr->Data;
+    }
+    Ptr = Ptr->Next;
+  }
+  return NULL;
+}
+
+SU_PList ParseProfile(const char S[])
+{
+  char *s,*p,*q;
+  SU_PList prof = NULL;
+
+  s = (char *)S;
+  p = strtok(s," ");
+  if(p == NULL)
+    return NULL;
+  q = strtok(NULL," ");
+  if(q == NULL)
+    return NULL;
+  prof = SU_AddElementTail(prof,strdup(p));
+  prof = SU_AddElementTail(prof,strdup(q));
+  if(DebugLevel >= 1)
+    printf("Loading Profile : %s = %s",p,q);
+  p = strtok(NULL," ");
+  while(p != NULL)
+  {
+    prof = SU_AddElementTail(prof,strdup(p));
+    if(DebugLevel >= 1)
+      printf(" %s",p);
+    p = strtok(NULL," ");
+  }
+  if(DebugLevel >= 1)
+    printf("\n");
+  return prof;
+}
+
+void LoadProfiles(const char FileName[])
+{
+  FILE *fp;
+  char S[1024];
+
+  if(SMS_Profiles != NULL)
+  {
+    SU_PList Ptr = SMS_Profiles;
+    while(Ptr != NULL)
+    {
+      SU_FreeListElem(Ptr->Data);
+      Ptr = Ptr->Next;
+    }
+    SU_FreeListElem(SMS_Profiles);
+    SMS_Profiles = NULL;
+  }
+  fp = fopen(FileName,"rt");
+  if(fp == NULL)
+    return;
+  while(SU_ReadLine(fp,S,sizeof(S)))
+  {
+    SU_PList prof;
+
+    if((S[0] == '#') || (S[0] == 0))
+      continue;
+    prof = ParseProfile(S);
+    if(prof != NULL)
+      SMS_Profiles = SU_AddElementHead(SMS_Profiles,prof);
+  }
+  fclose(fp);
+}
+
 void LoadAliases(const char FileName[])
 {
   FILE *fp;
@@ -1550,7 +1878,7 @@ void PrintProviderHelp(const char ProviderName[],SMS_PProvider Pv)
 {
   int i;
 
-  printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'01\n",SMSSEND_VERSION);
+  printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'03\n",SMSSEND_VERSION);
   printf("Arguments for provider %s :\n",ProviderName);
   for(i=0;i<Pv->NbParams;i++)
   {
@@ -1568,7 +1896,7 @@ void PrintProviderHelp(const char ProviderName[],SMS_PProvider Pv)
 
 void PrintHelp(char *Msg,bool version)
 {
-  printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'01\n",SMSSEND_VERSION);
+  printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'03\n",SMSSEND_VERSION);
   if(version)
     return;
   printf("Usage : smssend [options] <provider> [arguments] -- [SkyUtils options]\n");
@@ -1577,6 +1905,7 @@ void PrintHelp(char *Msg,bool version)
   printf("           %s\n",SU_GetOptionsString());
   printf("        * Use -help as the first provider option to get required\n          fields for this provider\n");
   printf("        * Use -update as the first provider option to search for\n          a new version of this script\n");
+  printf("        * Use -install as the first provider option to download\n           and install this script\n");
   if(Msg != NULL)
     printf("\n%s\n",Msg);
 }
@@ -1600,9 +1929,11 @@ char *ReadParamFromStdin()
 int main(int argc,char *argv[])
 {
   SMS_PProvider Pv;
-  int i,pvPos;
-  char PvName[512],AName[512];
+  int i,j,pvPos;
+  char PvName[512],AName[512],PName[512];
   char Cmd[1024];
+  int Nargc = 1;
+  char *Nargv[50];
 #ifdef _WIN32
   float fl; // This float is needed to force VC to link the math lib.. VC bug ;(
 
@@ -1614,7 +1945,7 @@ int main(int argc,char *argv[])
     printf("SmsSend error : Couldn't find a usable WinSock dll\n");
     return -1;
   }
-#endif
+#endif /* _WIN32 */
 
   argc = SU_GetSkyutilsParams(argc,argv);
   if(argc == 1)
@@ -1623,17 +1954,25 @@ int main(int argc,char *argv[])
     return -3;
   }
   DebugLevel = SU_GetDebugLevel();
+#ifdef _WIN32
+  snprintf(PName,sizeof(PName),"%s\\profiles",SMSSEND_SHAREPATH);
+#else /* !_WIN32 */
+  snprintf(PName,sizeof(PName),"%s/.smssend/profiles",getenv("HOME"));
+#endif /* _WIN32 */
+  LoadProfiles(PName);
   /* Parse options */
-  i = 1;
+  i = j = 1;
+  Nargv[0] = argv[0];
   while(i < argc)
   {
+    j++;
     if(argv[i][0] == '-')
     {
       if(strcasecmp(argv[i],"-update") == 0)
       {
         if(!SMS_Quiet)
         {
-          printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'01\n",SMSSEND_VERSION);
+          printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'03\n",SMSSEND_VERSION);
           printf("Trying to update SmsSend...\n");
         }
         CheckForUpdateSmsSend();
@@ -1668,42 +2007,74 @@ int main(int argc,char *argv[])
         return -3;
       }
     }
+    else if(argv[i][0] == '%') /* Search for profile */
+    {
+      SU_PList Prof = GetProfile((const char *)&argv[i][1]);
+      if(Prof == NULL)
+      {
+        printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'03\n",SMSSEND_VERSION);
+        printf("Unknown profile '%s'\n",(char *)&argv[i][1]);
+        return -3;
+      }
+      Prof = Prof->Next;
+      while(Prof != NULL)
+      {
+        Nargv[Nargc++] = (char *)Prof->Data;
+        Prof = Prof->Next;
+      }
+      j--;
+    }
     else
-      break;
+    {
+      Nargv[Nargc++] = argv[i];
+      j--;
+    }
     i++;
   }
-  if(i >= argc)
+  i = j;
+  if(i >= Nargc)
   {
     PrintHelp("Provider not specified.",false);
     return -3;
   }
-  if(strstr(argv[i],".sms") == NULL)
-    snprintf(PvName,sizeof(PvName),"%s.sms",argv[i]);
+  if(strstr(Nargv[i],".sms") == NULL)
+    snprintf(PvName,sizeof(PvName),"%s.sms",Nargv[i]);
   else
-    strcpy(PvName,argv[i]);
+    strcpy(PvName,Nargv[i]);
+  /* Check for a new script to install */
+  if(((i+1) < Nargc) && (strcasecmp(Nargv[i+1],"-install") == 0))
+  {
+    if(!SMS_Quiet)
+    {
+      printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'03\n",SMSSEND_VERSION);
+      printf("Trying to install %s script...\n",PvName);
+    }
+    InstallScript(PvName);
+    return 0;
+  }
   /* First try to load local .sms script */
   Pv = LoadProviderFile(PvName);
   if(Pv == NULL)
   {
 #ifndef _WIN32
     /* Then try to load user script */
-    if(strstr(argv[i],".sms") == NULL)
-      snprintf(PvName,sizeof(PvName),"%s/.smssend/%s.sms",getenv("HOME"),argv[i]);
+    if(strstr(Nargv[i],".sms") == NULL)
+      snprintf(PvName,sizeof(PvName),"%s/.smssend/%s.sms",getenv("HOME"),Nargv[i]);
     else
-      snprintf(PvName,sizeof(PvName),"%s/.smssend/%s",getenv("HOME"),argv[i]);
+      snprintf(PvName,sizeof(PvName),"%s/.smssend/%s",getenv("HOME"),Nargv[i]);
     Pv = LoadProviderFile(PvName);
     if(Pv == NULL)
-#endif
+#endif /* !_WIN32 */
     {
       /* Then try to load global shared scripts */
-      if(strstr(argv[i],".sms") == NULL)
-        snprintf(PvName,sizeof(PvName),"%s/%s.sms",SMSSEND_SHAREPATH,argv[i]);
+      if(strstr(Nargv[i],".sms") == NULL)
+        snprintf(PvName,sizeof(PvName),"%s/%s.sms",SMSSEND_SHAREPATH,Nargv[i]);
       else
-        snprintf(PvName,sizeof(PvName),"%s/%s",SMSSEND_SHAREPATH,argv[i]);
+        snprintf(PvName,sizeof(PvName),"%s/%s",SMSSEND_SHAREPATH,Nargv[i]);
       Pv = LoadProviderFile(PvName);
       if(Pv == NULL)
       {
-        snprintf(Cmd,sizeof(Cmd),"Cannot load provider file (not found in ./ , ~/.smssend or %s/) : %s%s\n",SMSSEND_SHAREPATH,argv[i],(strstr(argv[i],".sms") != NULL)?"":".sms");
+        snprintf(Cmd,sizeof(Cmd),"Cannot load provider file (not found in ./ , ~/.smssend or %s/) : %s%s\n",SMSSEND_SHAREPATH,Nargv[i],(strstr(Nargv[i],".sms") != NULL)?"":".sms");
         PrintHelp(Cmd,false);
         return -4;
       }
@@ -1711,52 +2082,52 @@ int main(int argc,char *argv[])
   }
   pvPos = i;
   i++;
-  if(i >= argc)
+  if(i >= Nargc)
   {
-    PrintProviderHelp(argv[pvPos],Pv);
+    PrintProviderHelp(Nargv[pvPos],Pv);
     return -6;
   }
-  if(strcasecmp(argv[i],"-help") == 0)
+  if(strcasecmp(Nargv[i],"-help") == 0)
   {
-    PrintProviderHelp(argv[pvPos],Pv);
+    PrintProviderHelp(Nargv[pvPos],Pv);
     return -5;
   }
-  else if(strcasecmp(argv[i],"-update") == 0)
+  else if(strcasecmp(Nargv[i],"-update") == 0)
   {
     if(!SMS_Quiet)
     {
-      printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'01\n",SMSSEND_VERSION);
-      printf("Trying to update %s script...\n",argv[pvPos]);
+      printf("SmsSend version %s - Copyright(c) Ze KiLleR / SkyTech - 2000'03\n",SMSSEND_VERSION);
+      printf("Trying to update %s script...\n",Nargv[pvPos]);
     }
     CheckForUpdate(PvName);
     return 0;
   }
-  if(argc < (Pv->NbParams+pvPos+1))
+  if(Nargc < (Pv->NbParams+pvPos+1))
   {
     PrintHelp("Not enough arguments for this provider. Try -help as first provider argument.",false);
     return -6;
   }
-  else if(argc > (Pv->NbParams+pvPos+1))
+  else if(Nargc > (Pv->NbParams+pvPos+1))
   {
     PrintHelp("Too many arguments for this provider. Try -help as first provider argument.",false);
     return -6;
   }
 #ifdef _WIN32
   snprintf(AName,sizeof(AName),"%s\\aliases",SMSSEND_SHAREPATH);
-#else
+#else /* !_WIN32 */
   snprintf(AName,sizeof(AName),"%s/.smssend/aliases",getenv("HOME"));
-#endif
+#endif /* _WIN32 */
   LoadAliases(AName);
   for(i=0;i<Pv->NbParams;i++)
   {
-    if(strcmp(argv[i+pvPos+1],"-") == 0)
+    if(strcmp(Nargv[i+pvPos+1],"-") == 0)
     {
       if(!SMS_Quiet)
         printf("Enter parameter %s :\n",Pv->Params[i].Name);
       Pv->Params[i].Value = ReadParamFromStdin();
     }
     else
-      Pv->Params[i].Value = strdup(SearchAlias(argv[i+pvPos+1]));
+      Pv->Params[i].Value = strdup(SearchAlias(Nargv[i+pvPos+1]));
   }
 
   ExecuteProvider(Pv);
@@ -1765,4 +2136,91 @@ int main(int argc,char *argv[])
 
   return 0;
 }
-#endif
+#endif /* __unix__ */
+
+#ifdef USE_PCRE
+#ifdef _WIN32
+#include "pcre.h"
+#else /* !_WIN32 */
+#include <pcre.h>
+#endif /* _WIN32 */
+
+/** this should be always multiple of 3 */
+#define OVECTOR_SIZE  120
+
+/*
+ * PM_GetRegexMatch -- find and return specified regex match.
+ *
+ *    data    -- input source
+ *    regex   -- matching regex
+ *    gpos    -- which global match to return
+ *    pos     -- which single match to return
+ *    buf     -- output buffer
+ *    blen    -- output buffer size
+ *
+ *
+ *  Function repeats pattern matching gpos-times, than copies
+ *  pos-th match to the buf buffer. On success returns 0 else
+ *  non-zero value.
+ */
+
+static int PM_GetRegexMatch (char *data, char *regex, int gpos, int pos, char *buf, int blen)
+{
+  pcre *pcre;
+  const char *error;
+  int erroffset;
+  int rc, c, dpos;
+  int ovector[OVECTOR_SIZE];
+
+  /* invalid input */
+  if (data == NULL || regex == NULL || buf == NULL || blen < 0)
+    return 1;
+
+  /* indexes out of bounds */
+  if (gpos < 1 || pos < 1 || pos+1 >= OVECTOR_SIZE/3)
+    return 1;
+
+  /* compile regular expression, no option support yet */
+  pcre = pcre_compile (regex, 0, &error, &erroffset, NULL);
+  if (pcre == NULL)
+    {
+#ifdef DEBUG
+      printf ("RTRegex failed: %s: %s\n", error ? error : "",
+	      erroffset > 0 ? data + erroffset : "");
+#endif /* DEBUG */
+      return 1;
+    }
+
+  rc = -1;
+  dpos = 0;
+  for (c = 0; c < gpos; c++)
+    {
+      rc =
+	pcre_exec (pcre, NULL, data, strlen (data), dpos, 0, ovector,
+		   OVECTOR_SIZE);
+      if (rc < 0)
+	{
+	  break;
+	}
+      dpos = ovector[1] + 1;
+    }
+
+  pcre_free (pcre);
+
+  if (rc > 0)
+    {
+      int start = ovector[pos * 2], stop = ovector[pos * 2 + 1];
+      if (start != -1 && stop != -1)
+	{
+	  int len = stop - start;
+          if (len > 0) {
+	    len = (len < blen) ? len : blen - 1;
+	    strncpy (buf, data + start, len);
+	    buf[len] = 0;
+	    return 0;
+          }
+	}
+    }
+  return 1;
+}
+#endif /* USE_PCRE */
